#include "cube.h"
#include "effect.h"
#include "draw.h"
#include "config.h"
#include "pt_cornell_1_2.h"
#include <stdlib.h>
#include <plib.h>
#include <xc.h>
#define dTime_ms PBCLK/2000
#define dTime_us PBCLK/2000000
#define PBCLK 40000000 // peripheral bus clock
#define SYS_FREQ 40000000


volatile unsigned char current_layer;

int speed = 500;        // game speed ms
unsigned char key= 0;   // key you pressed 
unsigned char data;     // intermediate state for key
int  gameOver = 0;      // game over flag
unsigned char off = 0b00000000; 

int  dx[6] = {1, -1, 0, 0, 0, 0};  //x,y,z direction
int  dy[6] = {0, 0, 1, -1, 0, 0};  //0 right,1 left,2 forward,3 backward,4 up,5 down
int  dz[6] = {0, 0, 0, 0, 1, -1};

int  snakeLength = 1;
struct Snake {  // one node of snake 
 int x, y, z;  
 int heading;   // current heading direction
} Snake[300];   // i doubt anyone can do more than this


void initial();    
void show();          // move fb[][] to cube[][] to refresh
void create_food();   // random food
void button();        // process the button pressed  
void move();          // move the snake
void check_crash();   
void check_head(int x, int y, int z);  // check status of head 

void open();          // one open animation
void string();        // SNAKE GO animation
void delay_us(unsigned long i);   // for polling controller
void launch_effect (int effect);  
void launch_strings(int index);


int start =0;
int reset =0;

int effect_index= -1;
int speed_index = 0;
int ChangeSpeed1,ChangeSpeed0;  //record option button on NES


int lightUpOrder[8]={0, 1, 3, 2, 6, 7, 5, 4}; // need to light up in this order

static struct pt pt_snake;
static struct pt pt_control;

//======================= Snake Thread =============================================================
static PT_THREAD (protothread_snake(struct pt *pt))
{
   PT_BEGIN(pt);

    initial();  
    //show(); 
    
    while( start == 1)  
 {  

  PT_YIELD_TIME_msec(speed);

  button();  // process button

if (reset == 1)  {  // reset process
    reset = 0;
    gameOver =0;
    snakeLength = 1;
    initial();

}
  
  move();   // move the snake

  if(gameOver)  // crash into walls or self play effects
  {   
      PT_YIELD_TIME_msec(10);
      effect_index = effect_index +1;
      launch_effect(effect_index);
      fill(0x00);
      if (effect_index>12)
          effect_index=-1;
      }
 
  // copy fb to cube to refresh 
  else 
  memcpy(cube, fb, 64);

 }  
    
   PT_END(pt);
}
//===================== Control Thread =========================================================
static PT_THREAD (protothread_control(struct pt *pt))
{
   PT_BEGIN(pt); 
   int i;  
   
   mPORTBClearBits(BIT_5);  // latch low
   mPORTASetBits(BIT_2);    // clock low

   while(1){

         PT_YIELD_TIME_msec(10);
         // clear up
         data = 0b00000000;
         mPORTBClearBits(BIT_5); // latch low
         mPORTAClearBits(BIT_2); // clock low     
         
        // Start transmitting
         mPORTBSetBits(BIT_5); // latch high
         delay_us(3);
         mPORTBClearBits(BIT_5); // latch low
    
         // something weird happened when I tried to use for loop
         // so instead I write up 8 times
         // 0     
        if( mPORTAReadBits(BIT_4) == 0)
            data = 0;
        else
            data = 1;
         
         delay_us(3);
         //PT_YIELD_TIME_msec(2);

         //1
         mPORTASetBits(BIT_2); // clock high
         delay_us(1);   
         data =data << 1;
         if( mPORTAReadBits(BIT_4) == 0)
            data =  data + 0;
         else
            data =  data + 1;
         delay_us(1);
         mPORTAClearBits(BIT_2); // clock low
         delay_us(3);
         
         //2
         mPORTASetBits(BIT_2); // clock high
         delay_us(1);
         data =data << 1;        
         if( mPORTAReadBits(BIT_4) == 0)
            data =  data + 0;
         else
            data =  data + 1;
         delay_us(1);
         mPORTAClearBits(BIT_2); // clock low
         delay_us(3);
         
         //3
         mPORTASetBits(BIT_2); // clock high
         delay_us(1);
         data =data << 1;
          if( mPORTAReadBits(BIT_4) == 0)
            data =  data + 0;
         else
            data =  data + 1;
         delay_us(1);
         mPORTAClearBits(BIT_2); // clock low
         delay_us(3);
         
         //4
        mPORTASetBits(BIT_2); // clock high
         delay_us(1); 
         data =data << 1;
          if( mPORTAReadBits(BIT_4) == 0)
            data =  data + 0;
         else
            data =  data + 1;
         delay_us(1);
         mPORTAClearBits(BIT_2); // clock low
         delay_us(3);
         
         //5
         mPORTASetBits(BIT_2); // clock high
         delay_us(1);
         data =data << 1;
          if( mPORTAReadBits(BIT_4) == 0)
            data =  data + 0;
         else
            data =  data + 1;
         delay_us(1);
         mPORTAClearBits(BIT_2); // clock low
         delay_us(3);
         
         //6
         mPORTASetBits(BIT_2); // clock high
         delay_us(1); 
         data =data << 1;
          if( mPORTAReadBits(BIT_4) == 0)
            data =  data + 0;
         else
            data =  data + 1;
         delay_us(1);
         mPORTAClearBits(BIT_2); // clock low
         delay_us(3);
         
         //7
        mPORTASetBits(BIT_2); // clock high
         delay_us(1);
         data =data << 1;
          if( mPORTAReadBits(BIT_4) == 0)
            data =  data + 0;
         else
            data =  data + 1;
         delay_us(1);
         mPORTAClearBits(BIT_2); // clock low
         delay_us(3);

         
        // to change the speed
        if (data == 0b11011111){
           ChangeSpeed1=1; 
        }
        else
           ChangeSpeed1 =0;
         
        if(ChangeSpeed1 == 0 && ChangeSpeed0 == 1){
            speed_index = speed_index +1;
            if (speed_index == 3)
                speed_index=0;
            switch(speed_index){
                case 0: speed = 500; break;
                case 1: speed = 1000; break;
                case 2: speed = 250; break;
            }          
        }
        ChangeSpeed0 = ChangeSpeed1;    
         
                
        // check reset 
        if (data == 0b11101111){
           reset =1; 
        }
        else
           reset =0;
         

         // record in key when anything pressed
         if (data != 0b11111111){
             start =1;
            key = data;
            srand(ReadTimer2());
         }
         
         
   }
   
   PT_END(pt);
}


void delay_us(unsigned long i){
/* Create a software delay about i us long
 * Parameters:
 *      i:  equal to number of microseconds for delay
 * Returns: Nothing
 * Note: Uses Core Timer. Core Timer is cleared at the initialiazion of
 *      this function. So, applications sensitive to the Core Timer are going
 *      to be affected
 */
    unsigned int j;
    j = dTime_us * i;
    WriteCoreTimer(0);
    while (ReadCoreTimer() < j);
}


// =========================== ISR ==================================================================

void __ISR(_TIMER_2_VECTOR, ipl2) Timer2Handler2(void)
{
    mT2ClearIntFlag();
    
	int i, j, addr, addr2;

    // clean up first
    for (j = 0 ; j < 8; j++) 
		{		
        addr2 = lightUpOrder[j];
				switch (addr2){
					case 0 : mPORTBClearBits(BIT_7); break;
					case 1 : mPORTBClearBits(BIT_8); break;
					case 2 : mPORTBClearBits(BIT_9); break;
					case 3 : mPORTBClearBits(BIT_10);break;
					case 4 : mPORTBClearBits(BIT_11);break;
					case 5 : mPORTBClearBits(BIT_13);break;
					case 6 : mPORTBClearBits(BIT_14);break;
					case 7 : mPORTBClearBits(BIT_15);break;
					}
        }
    
    
    // apply to 8 latches
    for (i = 0; i < 8; i++)
	{      
	switch (i) {
			case 0 : mPORTBClearBits(BIT_0); mPORTAClearBits(BIT_1); mPORTAClearBits(BIT_0); break;
			case 1 : mPORTBClearBits(BIT_0); mPORTAClearBits(BIT_1); mPORTASetBits(BIT_0);   break;
			case 2 : mPORTBClearBits(BIT_0); mPORTASetBits(BIT_1);   mPORTAClearBits(BIT_0); break;
			case 3 : mPORTBClearBits(BIT_0); mPORTASetBits(BIT_1);   mPORTASetBits(BIT_0);   break;
			case 4 : mPORTBSetBits(BIT_0);   mPORTAClearBits(BIT_1); mPORTAClearBits(BIT_0); break;
			case 5 : mPORTBSetBits(BIT_0);   mPORTAClearBits(BIT_1); mPORTASetBits(BIT_0);   break;
			case 6 : mPORTBSetBits(BIT_0);   mPORTASetBits(BIT_1);   mPORTAClearBits(BIT_0); break;
			case 7 : mPORTBSetBits(BIT_0);   mPORTASetBits(BIT_1);   mPORTASetBits(BIT_0);   break;
		}		
    }
    
    
    

	// select which layer is currently working on 
    switch (current_layer) {
		case 0 : mPORTBClearBits(BIT_3); mPORTBClearBits(BIT_2); mPORTBClearBits(BIT_1); break;
		case 1 : mPORTBClearBits(BIT_3); mPORTBClearBits(BIT_2); mPORTBSetBits(BIT_1);   break;
		case 2 : mPORTBClearBits(BIT_3); mPORTBSetBits(BIT_2);   mPORTBClearBits(BIT_1); break;
		case 3 : mPORTBClearBits(BIT_3); mPORTBSetBits(BIT_2);   mPORTBSetBits(BIT_1);   break;
		case 4 : mPORTBSetBits(BIT_3);   mPORTBClearBits(BIT_2); mPORTBClearBits(BIT_1); break;
		case 5 : mPORTBSetBits(BIT_3);   mPORTBClearBits(BIT_2); mPORTBSetBits(BIT_1);   break;
		case 6 : mPORTBSetBits(BIT_3);   mPORTBSetBits(BIT_2);   mPORTBClearBits(BIT_1); break;
		case 7 : mPORTBSetBits(BIT_3);   mPORTBSetBits(BIT_2);   mPORTBSetBits(BIT_1);   break;
    }
    
  // light up in this order  
	for (i = 0; i < 8; i++)
	{
		addr = lightUpOrder[i];
        
		switch (addr) {
			case 0 : mPORTBClearBits(BIT_0); mPORTAClearBits(BIT_1); mPORTAClearBits(BIT_0); break;
			case 1 : mPORTBClearBits(BIT_0); mPORTAClearBits(BIT_1); mPORTASetBits(BIT_0);   break;
			case 2 : mPORTBClearBits(BIT_0); mPORTASetBits(BIT_1);   mPORTAClearBits(BIT_0); break;
			case 3 : mPORTBClearBits(BIT_0); mPORTASetBits(BIT_1);   mPORTASetBits(BIT_0);   break;
			case 4 : mPORTBSetBits(BIT_0);   mPORTAClearBits(BIT_1); mPORTAClearBits(BIT_0); break;
			case 5 : mPORTBSetBits(BIT_0);   mPORTAClearBits(BIT_1); mPORTASetBits(BIT_0);   break;
			case 6 : mPORTBSetBits(BIT_0);   mPORTASetBits(BIT_1);   mPORTAClearBits(BIT_0); break;
			case 7 : mPORTBSetBits(BIT_0);   mPORTASetBits(BIT_1);   mPORTASetBits(BIT_0);   break;
		}

    //data of this latch
		int value = cube[current_layer][addr];

		for (j = 0 ; j < 8; j++) 
		{
			if ((value >> j) % 2 == 1)
				switch (j){
					case 0 : mPORTBSetBits(BIT_7); break;
					case 1 : mPORTBSetBits(BIT_8); break;
					case 2 : mPORTBSetBits(BIT_9); break;
					case 3 : mPORTBSetBits(BIT_10);break;
					case 4 : mPORTBSetBits(BIT_11);break;
					case 5 : mPORTBSetBits(BIT_13);break;
					case 6 : mPORTBSetBits(BIT_14);break;
					case 7 : mPORTBSetBits(BIT_15);break;
					}
			else if ((value >> j) % 2 == 0)
				switch (j){
					case 0 : mPORTBClearBits(BIT_7) ;break;
					case 1 : mPORTBClearBits(BIT_8); break;
					case 2 : mPORTBClearBits(BIT_9); break;
					case 3 : mPORTBClearBits(BIT_10);break;
					case 4 : mPORTBClearBits(BIT_11);break;
					case 5 : mPORTBClearBits(BIT_13);break;
					case 6 : mPORTBClearBits(BIT_14);break;
					case 7 : mPORTBClearBits(BIT_15);break;
					}
		}

	}

  // increase by one to refernece next layer
	current_layer++;
	if (current_layer == 8)
		current_layer = 0;
}


//============================== main =================================================================
// Main loop
int main (void)
{
   current_layer = 0;
   ANSELA = 0;
   ANSELB = 0;
   // Set up system wide interrupt
   INTEnableSystemMultiVectoredInt();
   // config threads
   PT_setup();
   
   // set the corresponding IO ports to output
   // group A
   mPORTASetPinsDigitalOut(BIT_0 | BIT_1 | BIT_2 | BIT_3 );
   mPORTASetPinsDigitalIn(BIT_4);
   
   // group B
   mPORTBSetPinsDigitalOut(BIT_0 | BIT_1 | BIT_2 | BIT_3 |BIT_5 | BIT_7 | BIT_8 | BIT_9 | BIT_10 | BIT_11 | BIT_13 | BIT_14 | BIT_15);
   

   OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, 20000);  // sample rate = 40MHz/2000=2kHz  
   ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);     		  // set up the timer interrupt with a priority of 2
   mT2ClearIntFlag(); 									                  // and clear the interrupt flag

   // init the threads
   PT_INIT(&pt_snake);
   PT_INIT(&pt_control);
    
   // open animation
   open();
   // SNAKE GO
   string();
              
   while(1){
      PT_SCHEDULE(protothread_timer(&pt_snake));
      PT_SCHEDULE(protothread_c(&pt_control));
   }

	
}



// ================================ helper functions ==============================================
void string(){
    int i,j; 
    delay_ms(8000);
    fill(0x00);
    for (i=0;i<=9;i++){
        launch_strings(i);
        for(j=0;j<7;j++){
            delay_ms(200);
            shift(AXIS_Y, -1);
        }
        delay_ms(1500);
    }   
}

void launch_strings(int index){   
    switch(index)
    {   
        
        // S
        case 0:   
            setvoxel(2,7,1); setvoxel(3,7,1); setvoxel(4,7,1); setvoxel(1,7,2);
            setvoxel(5,7,2); setvoxel(2,7,3); setvoxel(3,7,4); setvoxel(4,7,5);
            setvoxel(5,7,6); setvoxel(1,7,6); setvoxel(2,7,7); setvoxel(3,7,7);
            setvoxel(4,7,7);
            break;
        // N
        case 1:
            setvoxel(1,7,1); setvoxel(1,7,2); setvoxel(1,7,3); setvoxel(1,7,4);
            setvoxel(1,7,5); setvoxel(1,7,6); setvoxel(2,7,2); setvoxel(3,7,3);
            setvoxel(4,7,4); setvoxel(5,7,5); setvoxel(6,7,6); setvoxel(6,7,5);
            setvoxel(6,7,4); setvoxel(6,7,3); setvoxel(6,7,2); setvoxel(6,7,1);
            break;
        // A
        case 2:
            setvoxel(3,7,0); setvoxel(2,7,2);  setvoxel(1,7,4);
            setvoxel(0,7,6); setvoxel(4,7,2); setvoxel(5,7,4); setvoxel(6,7,6);
            setvoxel(2,7,4); setvoxel(3,7,4); setvoxel(4,7,4);
            break;
        // K
        case 3:
            setvoxel(2,7,1); setvoxel(2,7,2); setvoxel(2,7,3); setvoxel(2,7,4);
            setvoxel(2,7,5); setvoxel(2,7,6); setvoxel(5,7,1); setvoxel(4,7,2);
            setvoxel(3,7,3); setvoxel(4,7,4); setvoxel(5,7,5); setvoxel(6,7,6);
            break;
        // E
        case 4:
            setvoxel(2,7,1); setvoxel(2,7,2); setvoxel(2,7,3); setvoxel(2,7,4);
            setvoxel(2,7,5); setvoxel(2,7,6); setvoxel(2,7,7); setvoxel(3,7,1);
            setvoxel(4,7,1); setvoxel(5,7,1); setvoxel(3,7,4); setvoxel(4,7,4);
            setvoxel(5,7,4); setvoxel(3,7,7); setvoxel(4,7,7); setvoxel(5,7,7);
            break;
        // nothing
            case 5:
                fill(0x00);
            break;
            
        // G 
            case 6:
            setvoxel(2,7,1); setvoxel(3,7,1); setvoxel(4,7,1); setvoxel(5,7,2);
            setvoxel(1,7,2); setvoxel(1,7,3); setvoxel(1,7,4); setvoxel(1,7,5);
            setvoxel(2,7,6); setvoxel(3,7,6); setvoxel(4,7,6); setvoxel(4,7,5);
            setvoxel(4,7,4); setvoxel(3,7,4); setvoxel(5,7,4); clrvoxel(5,7,5);
            clrvoxel(5,7,3); 
            break;
            
        // O 
            case 7:
            setvoxel(2,7,1); setvoxel(3,7,1); setvoxel(4,7,1); setvoxel(5,7,2);
            setvoxel(1,7,2); setvoxel(1,7,3); setvoxel(1,7,4); setvoxel(1,7,5);
            setvoxel(2,7,6); setvoxel(3,7,6); setvoxel(4,7,6); setvoxel(5,7,5);
            setvoxel(5,7,4); setvoxel(5,7,3); 
            break;
        // smile
            case 8:
                fill(0x00);
            break;  
         // smile   
            case 9:
            //setvoxel(2,7,1); 
            setvoxel(2,7,2); setvoxel(2,7,3); 
            //setvoxel(5,7,1);
            setvoxel(5,7,2); setvoxel(5,7,3); setvoxel(0,7,5);
            setvoxel(7,7,5);
            setvoxel(1,7,6); setvoxel(2,7,7); setvoxel(3,7,7); setvoxel(4,7,7);
            setvoxel(5,7,7); setvoxel(6,7,6); 
            break;                            
    }
    
}


void open(){
    
    int i;
    	effect_planboing (AXIS_Z, 250);
			effect_planboing (AXIS_X, 250);
			effect_planboing (AXIS_Y, 250);                      
}



void launch_effect (int effect)
{
  int i;
    unsigned char ii;

  fill(0x00);

  switch (effect)
  {
    case 0x00: 
      effect_rain(100);
      break;
         
    case 1:
      fill(0x00);
      effect_telcstairs(0,200,0xff);
      effect_telcstairs(0,200,0x00);
      effect_telcstairs(1,200,0xff);
      effect_telcstairs(1,200,0x00);
      break;
        
    case 2:
      effect_boxside_randsend_parallel (AXIS_Z, 0 , 150,2);
      delay_ms(200);

      effect_boxside_randsend_parallel (AXIS_Y, 1 , 150,2);
      delay_ms(200);
      
      effect_boxside_randsend_parallel (AXIS_X, 0 , 150,2);
      delay_ms(200);  
      break;
        
    case 3:
      effect_random_sparkle();
      break;
        
    case 4: 
      effect_wormsqueeze (2, AXIS_Z, -1, 100, 300);
      break;
        
    case 5: 
      effect_blinky2();
      break;
        
    case 6: 
      effect_loadbar(350);
               
    case 7:
      effect_planboing (AXIS_Z, 250);
      effect_planboing (AXIS_X, 250);
      effect_planboing (AXIS_Y, 250);   
      break;
    
    case 8:
      fill(0x00);
      effect_telcstairs(0,200,0xff);
      effect_telcstairs(0,200,0x00);
      effect_telcstairs(1,200,0xff);
      effect_telcstairs(1,200,0x00);
      break;
      
    case 9:
      effect_axis_updown_randsuspend(AXIS_Z, 350,1000,0);
      //effect_axis_updown_randsuspend(AXIS_Z, 550,5000,1);
      effect_axis_updown_randsuspend(AXIS_X, 350,1000,0);
      //effect_axis_updown_randsuspend(AXIS_X, 550,5000,1);
      effect_axis_updown_randsuspend(AXIS_Y, 350,1000,0);
      //effect_axis_updown_randsuspend(AXIS_Y, 550,5000,1);
      break;
      
    case 10:
       for (ii=0;ii<8;ii++)
      {
        effect_box_shrink_grow (1, ii%4, ii & 0x04, 200);
      }

      effect_box_woopwoop(300,0);
      effect_box_woopwoop(300,1);
      effect_box_woopwoop(300,0);
      effect_box_woopwoop(300,1);
      break;
      break;
      
    case 11:
      boingboing(80, 300, 0x01, 0x02);
      break;
      
    case 12: 
      fill(0x00);
      effect_random_filler(3,1);
      effect_random_filler(3,0);
      effect_random_filler(3,1);
      effect_random_filler(3,0);
      effect_random_filler(3,1);
      effect_random_filler(3,0);
      effect_random_filler(3,1);
      effect_random_filler(3,0);
      break;

  }
}

// ====================== snake functions ==================================================================
void initial()  
{  
 int hx, hy, hz;  // random starting position of head
 tmpfill(0x00);   // clean up the fb

 hx = rand()%8;     
 hy = rand()%8;  
 hz = rand()%8;
 
 tmpsetvoxel(hx, hy, hz) ;  

 Snake[0].x = hx;  
 Snake[0].y = hy; 
 Snake[0].z = hz; 
 Snake[0].heading = 0;  

 create_food();   
 
}  


void create_food()  // generate one food 
{  
 int fx, fy, fz;  
 while(1)  
 {  
  fx = rand()%8;  
  fy = rand()%8;
  fz = rand()%8;     
  if(tmpgetvoxel(fx,fy,fz)!=1) { // ensure this position is untaken
   tmpsetvoxel(fx,fy,fz);
    break;  
  }  
 }  
}  


void button()    
{   
   
  switch(key)  
  {     
   case 0b11111110:   Snake[0].heading = 0; break;   // right -- NES controller
   case 0b11111101:   Snake[0].heading = 1; break;   // left
   case 0b01111111:   Snake[0].heading = 5; break;   // A
   case 0b10111111:   Snake[0].heading = 4; break;   // B
   case 0b11110111:   Snake[0].heading = 2; break;   // down
   case 0b11111011:   Snake[0].heading = 3; break;   // up
 
  }   
 
}  



void move()   // move the snake towards its heading direciton  
{  
 int i, headx, heady, headz;       // temporary store the coordinate of head
 int currentLength = snakeLength;  // temporary store the length of the snake
  
 headx = Snake[0].x;  
 heady = Snake[0].y;  
 headz = Snake[0].z; 
 tmpclrvoxel(headx, heady, headz); 
 
 // move the head to the next position
 Snake[0].x = Snake[0].x + dx[ Snake[0].heading ];  
 Snake[0].y = Snake[0].y + dy[ Snake[0].heading ];  
 Snake[0].z = Snake[0].z + dz[ Snake[0].heading ];


 check_crash();    
 check_head(headx, heady, headz);  // note: arguments here are the position of head before move


// Snake[0]is head, Snake[1] is tail, 
// Snake[snakeLength-1] is neck (the node after head)

 if(snakeLength == currentLength)       //no food picked, so move the body  
    for(i = 1; i < snakeLength; i++) {  //start moving the whole snake from the tail
      // for tail node, clear it 
      if(i == 1)    
        tmpclrvoxel(Snake[i].x, Snake[i].y, Snake[i].z);  
       
      // neck, the node right after head
      if(i == snakeLength-1) { 
        Snake[i].x = headx;  
        Snake[i].y = heady;
        Snake[i].z = headz;
        Snake[i].heading = Snake[0].heading;  
      }  
      else  {  
        Snake[i].x = Snake[i+1].x;  
        Snake[i].y = Snake[i+1].y;  
        Snake[i].z = Snake[i+1].z; 
        Snake[i].heading = Snake[i+1].heading;  
      }  
        
      tmpsetvoxel(Snake[i].x, Snake[i].y, Snake[i].z); 
   }  
}  


void check_crash()   
{  
    int i;
 // check if crash into the walls 
 if(Snake[0].x < 0 || Snake[0].x >= 8  
 || Snake[0].y < 0 || Snake[0].y >= 8
 || Snake[0].z < 0 || Snake[0].z >= 8
 )  
  gameOver = 1;  
 // check if crash into the body
 for (i = 1; i <snakeLength; i++){
    if (Snake[0].x == Snake[i].x &&
        Snake[0].y == Snake[i].y &&
        Snake[0].z == Snake[i].z )
      gameOver = 1;  
 }
}  


void check_head(int x, int y, int z)  // check head status after move  
{  
  // if new head positon is empty  
 if(tmpgetvoxel(Snake[0].x, Snake[0].y, Snake[0].z )== 0) {
   tmpsetvoxel( Snake[0].x ,Snake[0].y , Snake[0].z ) ;  // light up
 }
  // new head postion is food
 else if(tmpgetvoxel( Snake[0].x , Snake[0].y , Snake[0].z ) == 1) {      
      Snake[snakeLength].x = x;   // add the node as body  
      Snake[snakeLength].y = y;  
      Snake[snakeLength].z = z; 
      Snake[snakeLength].heading = Snake[0].heading;  
      tmpsetvoxel ( Snake[snakeLength].x ,  Snake[snakeLength].y , Snake[snakeLength].z );   
      snakeLength++;  
      create_food();  // since this food is taken, create a new one  
  }  
}








